@@ -99,6 +99,34 @@ class AutoSlugField(UniqueFieldMixin, SlugField):
     overwrite
         If set to True, overwrites the slug on every save (default: False)
 
+    slugify_function
+        Defines the function which will be used to "slugify" a content
+        (default: :py:func:`~django.template.defaultfilters.slugify` )
+
+    It is possible to provide custom "slugify" function with
+    the ``slugify_function`` function in a model class.
+
+    ``slugify_function`` function in a model class takes priority over
+    ``slugify_function`` given as an argument to :py:class:`~AutoSlugField`.
+
+    Example
+
+    .. code-block:: python
+
+        # models.py
+
+        from django.db import models
+
+        from django_extensions.db.fields import AutoSlugField
+
+
+        class MyModel(models.Model):
+            def slugify_function(self, content):
+                return content.replace('_', '-').lower()
+
+            title = models.CharField(max_length=42)
+            slug = AutoSlugField(populate_from='title')
+
     Inspired by SmileyChris' Unique Slugify snippet:
     http://www.djangosnippets.org/snippets/690/
     """
@@ -143,9 +171,10 @@ class AutoSlugField(UniqueFieldMixin, SlugField):
         value = re.sub('%s+' % re_sep, self.separator, value)
         return re.sub(r'^%s+|%s+$' % (re_sep, re_sep), '', value)
 
-    def slugify_func(self, content):
+    @staticmethod
+    def slugify_func(content, slugify_function):
         if content:
-            return self.slugify_function(content)
+            return slugify_function(content)
         return ''
 
     def slug_generator(self, original_slug, start):
@@ -179,10 +208,15 @@ class AutoSlugField(UniqueFieldMixin, SlugField):
         populate_from = self._populate_from
         if not isinstance(populate_from, (list, tuple)):
             populate_from = (populate_from, )
+
         slug_field = model_instance._meta.get_field(self.attname)
+        slugify_function = getattr(model_instance, 'slugify_function', self.slugify_function)
 
         # slugify the original field content and set next step to 2
-        slug_for_field = lambda lookup_value: self.slugify_func(self.get_slug_fields(model_instance, lookup_value))
+        slug_for_field = lambda lookup_value: self.slugify_func(
+            self.get_slug_fields(model_instance, lookup_value),
+            slugify_function=slugify_function
+        )
         slug = self.separator.join(map(slug_for_field, populate_from))
         start = 2
 
