@@ -1,8 +1,14 @@
 # -*- coding: utf-8 -*-
+
+from __future__ import print_function
+
+import sys
 import getpass
+import warnings
 
 from django.contrib.auth import get_user_model
 from django.core.management.base import BaseCommand, CommandError
+from django.utils.deprecation import RemovedInNextVersionWarning
 
 from django_extensions.management.utils import signalcommand
 
@@ -17,6 +23,10 @@ class Command(BaseCommand):
 
     @signalcommand
     def handle(self, *args, **options):
+        warn_msg = "`django_extensions.management.commands.passwd` is deprecated. You should use built-in `changepassword` django command instead"
+        print(self.style.ERROR(warn_msg), file=sys.stderr)
+        warnings.warn(warn_msg, RemovedInNextVersionWarning, stacklevel=2)
+
         username = options['username'] or getpass.getuser()
 
         User = get_user_model()
